﻿/*RecroGrid Framework DB v1.4.2 Oracle*/

DECLARE texists NUMBER;
BEGIN
	SELECT COUNT(1) INTO texists FROM USER_VIEWS WHERE VIEW_NAME='VRECROSEC_USERSINROLE';
	IF( texists = 0 ) THEN
		SELECT COUNT(1) INTO texists FROM USER_TABLES WHERE TABLE_NAME='ASPNETUSERROLES';
		IF( texists = 0 ) THEN
			EXECUTE IMMEDIATE 'CREATE OR REPLACE VIEW VRECROSEC_USER AS SELECT '''' AS USERID, '''' AS USERNAME FROM DUAL WHERE 1=2';
			EXECUTE IMMEDIATE 'CREATE OR REPLACE VIEW VRECROSEC_ROLE AS SELECT '''' AS ROLEID, '''' AS ROLENAME FROM DUAL WHERE 1=2';
			EXECUTE IMMEDIATE 'CREATE OR REPLACE VIEW VRECROSEC_USERSINROLE AS SELECT '''' AS USERID, '''' AS ROLEID FROM DUAL WHERE 1=2';
		ELSE
			EXECUTE IMMEDIATE 'CREATE OR REPLACE VIEW VRECROSEC_USER AS SELECT Id AS UserId, UserName FROM ASPNETUSERS';
			EXECUTE IMMEDIATE 'CREATE OR REPLACE VIEW VRECROSEC_ROLE AS SELECT Id AS RoleId, Name as RoleName FROM ASPNETROLES';
			EXECUTE IMMEDIATE 'CREATE OR REPLACE VIEW VRECROSEC_USERSINROLE AS SELECT UserId, RoleId FROM ASPNETUSERROLES';

			EXECUTE IMMEDIATE 'DECLARE texists NUMBER; BEGIN
			SELECT COUNT(1) into texists from ASPNETROLES;
			IF( texists = 0 ) THEN
				INSERT INTO ASPNETROLES(Id,Name) VALUES(1,''Administrators'');
				INSERT INTO ASPNETROLES(Id,Name) VALUES(2,''Users'');
			END IF; END;';
		END IF;
	END IF;
END;;
/

/*************************************************************************************************************************************************************************************/
CREATE OR REPLACE TRIGGER RECRODICT_STR_CHANGE 
AFTER DELETE OR INSERT OR UPDATE ON RECRODICT_STR 
BEGIN
  UPDATE RecroGrid_Framework SET ConfigValue = TO_CHAR(SYSTIMESTAMP,'YYYY-MM-DD HH24:MI:SSXFF') WHERE ConfigName='RecroDictLastUpdated';
END;;
/

CREATE OR REPLACE TRIGGER RECROGRID_ENTITY_CHANGE 
AFTER DELETE OR INSERT OR UPDATE ON RECROGRID_ENTITY 
BEGIN
  UPDATE RecroGrid_Framework SET ConfigValue = TO_CHAR(SYSTIMESTAMP,'YYYY-MM-DD HH24:MI:SSXFF') WHERE ConfigName='TSContextLastUpdated';
END;
/

CREATE OR REPLACE TRIGGER RECROGRID_PROPERTY_CHANGE 
AFTER DELETE OR INSERT OR UPDATE ON RECROGRID_PROPERTY 
BEGIN
  UPDATE RecroGrid_Framework SET ConfigValue = TO_CHAR(SYSTIMESTAMP,'YYYY-MM-DD HH24:MI:SSXFF') WHERE ConfigName='TSContextLastUpdated';
END;
/

CREATE OR REPLACE TRIGGER RECROGRID_OPTIONS_CHANGE 
AFTER DELETE OR INSERT OR UPDATE ON RECROGRID_OPTIONS 
BEGIN
  UPDATE RecroGrid_Framework SET ConfigValue = TO_CHAR(SYSTIMESTAMP,'YYYY-MM-DD HH24:MI:SSXFF') WHERE ConfigName='TSContextLastUpdated';
END;
/

CREATE OR REPLACE TRIGGER RECROTRACK_TABLE_CHANGE 
AFTER DELETE OR INSERT OR UPDATE ON RECROTRACK_TABLE 
BEGIN
  UPDATE RecroGrid_Framework SET ConfigValue = TO_CHAR(SYSTIMESTAMP,'YYYY-MM-DD HH24:MI:SSXFF') WHERE ConfigName='TSContextLastUpdated';
END;
/

CREATE OR REPLACE TRIGGER RECROGRID_MENU_CHANGE 
AFTER DELETE OR INSERT OR UPDATE ON RECROGRID_MENU
BEGIN
  UPDATE RecroGrid_Framework SET ConfigValue = TO_CHAR(SYSTIMESTAMP,'YYYY-MM-DD HH24:MI:SSXFF') WHERE ConfigName='RGFMenuLastUpdated';
END;
/
/*************************************************************************************************************************************************************************************/
CREATE OR REPLACE PROCEDURE RECROGRID_REPLACEID 
(
  ORIGENTITYID IN INT
, NEWENTITYID IN INT
) AS 
BEGIN

    EXECUTE IMMEDIATE 'alter trigger TR_RECROGRID_ENTITY disable';
    BEGIN
	INSERT INTO RECROGRID_ENTITY
		   (EntityId
		   ,EntityName
		   ,EntityVersion
		   ,TypeDefinition
		   ,Category
		   ,Title
		   ,EntityComment)
		SELECT 
			NEWENTITYID
			,Concat('_', e.EntityName)
			,e.EntityVersion
			,e.TypeDefinition
			,e.Category
			,e.Title
			,e.EntityComment FROM RECROGRID_ENTITY e WHERE e.EntityId=ORIGENTITYID;

	UPDATE RECROGRID_PROPERTY p SET p.EntityId=NEWENTITYID WHERE p.EntityId=ORIGENTITYID;
	UPDATE RECROGRID_PROPERTY p SET p.EntityBase=NEWENTITYID WHERE p.EntityBase=ORIGENTITYID;
	UPDATE RECROGRID_OPTIONS o SET o.EntityId=NEWENTITYID WHERE o.EntityId=ORIGENTITYID;

	DELETE FROM RECROGRID_ENTITY WHERE EntityId=ORIGENTITYID;
	UPDATE RECROGRID_ENTITY e SET e.EntityName=SUBSTR(e.EntityName,2) WHERE e.EntityId=NEWENTITYID;
    END;
    EXECUTE IMMEDIATE 'alter trigger TR_RECROGRID_ENTITY enable';

END RECROGRID_REPLACEID;
/

/*************************************************************************************************************************************************************************************/
--FEP
CREATE OR REPLACE VIEW VRECROSEC_FEP
AS
	SELECT f.FEPId, EntityId, FEPType, TransactionId, Key1
	FROM RECROSEC_FEP f inner join (
		SELECT MAX(FEPId) as FEPId, Key1
		FROM RECROTRACK_TRACK rtt 
			inner join RECROTRACK_OBJECT rto ON rtt.ObjectId = rto.ObjectId
			inner join RECROSEC_FEP fep ON rtt.TransactionId = fep.TransactionId
		GROUP BY Key1) t on f.FEPId = t.FEPId
/

--RecroSec
CREATE OR REPLACE VIEW VRECROSEC AS SELECT ObjectPermissionId, ObjectName, ObjectKey FROM RECROSEC_OBJECTPERMISSION
/

DECLARE texists NUMBER;
BEGIN
SELECT COUNT(1) INTO texists FROM user_tables WHERE table_name='VRECROSEC_BASEPERMISSION';
IF( texists <> 0 ) THEN
	EXECUTE IMMEDIATE 'DROP TABLE VRECROSEC_BASEPERMISSION';
END IF;
END;
/

CREATE OR REPLACE VIEW VRECROSEC_BASEPERMISSION (PermissionGroupId, "CREATEPERM", "READPERM", "UPDATEPERM", "DELETEPERM", rowtimestamp) AS 
	SELECT 
		PermissionGroupId,
		CAST(MAX(CASE WHEN PermissionId = 1 THEN Permission ELSE NULL END) as NUMBER(1)) as "CREATEPERM",
		CAST(MAX(CASE WHEN PermissionId = 2 THEN Permission ELSE NULL END) as NUMBER(1)) as "READPERM",
		CAST(MAX(CASE WHEN PermissionId = 4 THEN Permission ELSE NULL END) as NUMBER(1)) as "UPDATEPERM",
		CAST(MAX(CASE WHEN PermissionId = 8 THEN Permission ELSE NULL END) as NUMBER(1)) as "DELETEPERM",
		SYSDATE as "rowtimestamp" 
	FROM RECROSEC_PERMISSION
	GROUP BY PermissionGroupId
/

CREATE OR REPLACE FUNCTION FNRECROSEC_GETOBJECTPERMISSION 
(
  OBJECTNAME IN VARCHAR2 
, OBJECTKEY IN NVARCHAR2 
) RETURN INT AS 
	vObjectName varchar2(255) := OBJECTNAME;
	vObjectKey varchar2(128) := OBJECTKEY;
	vObjectPermissionId INT := null;
	idx INT;
BEGIN
    idx := INSTR(vObjectName, '/');
	if( idx > 0 and LENGTH(TRIM(TRANSLATE(vObjectKey, ' +-.0123456789', ' '))) is null ) then
		begin
			select p.ObjectPermissionId into vObjectPermissionId
				from recrosec_objectpermission p
				where p.ObjectName = vObjectName and p.Parent = vObjectKey;

			exception when no_data_found then vObjectPermissionId:= null;
		end;

		if( vObjectPermissionId is not null ) then
			return vObjectPermissionId;
		end if;

		if( vObjectKey is not null and LENGTH(TRIM(TRANSLATE(vObjectKey, ' +-.0123456789', ' '))) is null ) then
			return TO_NUMBER(vObjectKey);
		end if;

		--parent keresés
		vObjectName := SUBSTR(vObjectName, 1, idx - 1);
		return FNRECROSEC_GETOBJECTPERMISSION(vObjectName, '');
	end if;

	if( vObjectKey is not null ) then
		begin
			select p.ObjectPermissionId into vObjectPermissionId
				from recrosec_objectpermission p
				where p.ObjectName = vObjectName and p.ObjectKey = vObjectKey;

			exception when no_data_found then vObjectPermissionId:= null;
		end;

		if( vObjectPermissionId is not null ) then
			return vObjectPermissionId;
		end if;
	end if;

	begin
		select p.ObjectPermissionId into vObjectPermissionId
			from recrosec_objectpermission p
			where p.ObjectName = vObjectName and p.ObjectKey is null;

        exception when no_data_found then vObjectPermissionId:= null;
    end;

	return vObjectPermissionId;

END FNRECROSEC_GETOBJECTPERMISSION;
/

CREATE OR REPLACE PROCEDURE RECROSEC_ISOBJECTVALID 
(
  NAME IN VARCHAR2 
, TYPE IN VARCHAR2 
, VALID OUT INT
) AS 
BEGIN
  VALID := 1;
END RECROSEC_ISOBJECTVALID;
/

CREATE OR REPLACE PROCEDURE RECROSEC_GETOBJECTPERMISSION 
(
  OBJECTNAME IN VARCHAR2 
, OBJECTKEY IN NVARCHAR2 
, RES OUT SYS_REFCURSOR
) AS 
    vObjectPermissionId INT;
BEGIN

    vObjectPermissionId := FNRECROSEC_GETOBJECTPERMISSION(OBJECTNAME, OBJECTKEY);
    OPEN RES FOR SELECT p.* FROM RECROSEC_OBJECTPERMISSION p WHERE p.ObjectPermissionId = vObjectPermissionId;

END RECROSEC_GETOBJECTPERMISSION;
/

DECLARE texists NUMBER;
BEGIN
SELECT COUNT(1) INTO texists FROM user_tables WHERE table_name='TMP_RECROGRID_PERMISSION';
IF( texists <> 0 ) THEN
	EXECUTE IMMEDIATE 'DROP TABLE TMP_RECROGRID_PERMISSION';
END IF;
	EXECUTE IMMEDIATE 'CREATE GLOBAL TEMPORARY TABLE TMP_RECROGRID_PERMISSION(
		PermissionId NUMBER(10,0) NOT NULL,
		Permission NUMBER(1) NOT NULL,
		PRIMARY KEY (PermissionId)
	) ON COMMIT DELETE ROWS';
END;
/

DECLARE texists NUMBER;
BEGIN
SELECT COUNT(1) INTO texists FROM user_tables WHERE table_name='TMP_RECROGRID_INT';
IF( texists <> 0 ) THEN
	EXECUTE IMMEDIATE 'DROP TABLE TMP_RECROGRID_INT';
END IF;
	EXECUTE IMMEDIATE 'CREATE GLOBAL TEMPORARY TABLE TMP_RECROGRID_INT(
		Id NUMBER(10,0) NOT NULL,
		PRIMARY KEY (Id)
	) ON COMMIT DELETE ROWS';
END;
/

CREATE OR REPLACE PROCEDURE RECROSEC_GETPERMISSIONFORUSERA
(
  USERID IN NVARCHAR2
, OBJECTPERMISSIONID IN INT DEFAULT 0 
, OBJECTNAME IN VARCHAR2 DEFAULT NULL 
, OBJECTKEY IN NVARCHAR2 DEFAULT NULL 
, STORABLE OUT INT
, PREPROCESSING OUT VARCHAR2 
, POSTPROCESSING OUT VARCHAR2 
, ERRORCODE OUT INT
, NESTED IN INT DEFAULT 0 
, USERNAME IN VARCHAR2 DEFAULT NULL 
, RES OUT SYS_REFCURSOR
, RES2 OUT SYS_REFCURSOR
) AS 
    vObjectPermissionId int := OBJECTPERMISSIONID;
    origObjectPermissionId int;
    vUserId nvarchar2(128) := USERID;
	vUserName varchar2(255) := USERNAME;
    
	isObjectValid int;
	vStatement varchar2(2000);
    vStorable int := 0;
    
	vParent int := null;
	vInheritance int := 0;
	vPreprocessingTSQL varchar2(255) := null;
	vPostprocessingTSQL varchar2(255) := null;
	vPreprocessing1 varchar2(255) := null;
	vPostprocessing1 varchar2(255) := null;    

	ResA SYS_REFCURSOR;
	ResB SYS_REFCURSOR;
    
    l_exists NUMBER;
    
    CURSOR userPermissionGroup(vUserId nvarchar2, vObjectPermissionId varchar2) IS
		select g.permissionGroupId,op.inheritable
		from recrosec_permissiongroup g inner join
			 recrosec_oppg op on op.PermissionGroupId=g.PermissionGroupId inner join
			 recrosec_userpermission u on u.PermissionGroupId=g.PermissionGroupId
		where u.UserId=vUserId and op.ObjectPermissionId=vObjectPermissionId
		order by op.Priority;
    rec_c1 userPermissionGroup%rowtype;
        
    CURSOR rolePermissionGroup(vUserId nvarchar2, vObjectPermissionId int) IS
		select g.permissionGroupId,op.inheritable
		from RecroSec_PermissionGroup g inner join
			 recrosec_oppg op on op.PermissionGroupId=g.PermissionGroupId inner join
			 RecroSec_RolePermission r on g.PermissionGroupId=r.PermissionGroupId inner join
			 vRecroSec_UsersInRole ur on r.RoleId=ur.RoleId left outer join
			 RecroSec_RolePriority rp on r.RoleId=rp.RoleId
		where ur.UserId=vUserId and op.ObjectPermissionId=vObjectPermissionId
		order by NVL(rp.Priority,2147483647), op.Priority;        
    rec_c2 rolePermissionGroup%rowtype;
BEGIN

--1. PreprocessingTSQL, módosíthatja a kiindulási ObjectPermissionId-t (fa felépítésű adatok eseten hasznos)
--2. CustomUserPermission összegyűjtése
--3. UserPermission összegyűjtése priority szerint
--4. RolePermission összegyűjtése priority szerint
--5. Inheritance eseten parent alapján az 1. ponttól
--6. PostprocessingTSQL a #Permission módosítására

	ERRORCODE := 0;

	if( NVL(vObjectPermissionId,0) = 0 and OBJECTNAME is not null ) then
		vObjectPermissionId := FNRECROSEC_GETOBJECTPERMISSION(OBJECTNAME, OBJECTKEY);
	end if;

	if( NVL(vObjectPermissionId,0) = 0 ) then
		ERRORCODE := -1;
		if(NESTED = 0 ) then
			STORABLE := 0;
			PREPROCESSING := NULL;
			POSTPROCESSING := NULL;

			OPEN RES FOR SELECT 0 AS PermissionId, CAST(0 AS NUMBER(1)) as Permission from DUAL;
			OPEN RES2 FOR SELECT ERRORCODE AS ErrorCode, STORABLE AS Storable, PREPROCESSING AS Preprocessing, POSTPROCESSING AS Postprocessing from DUAL;
		end if;

		RETURN;
	end if;
    
	if( vUserId is null ) then
		select u.UserId into vUserId from vRecroSec_User u where u.UserName = vUserName;
	end if;
    
	if( NESTED = 0 ) then
        STORABLE := 1;
        PREPROCESSING := NULL;
        POSTPROCESSING := NULL;
	end if;
    
	select p.Parent, p.Inheritance, p.PreprocessingTSQL,  p.PostprocessingTSQL, p.Preprocessing,  p.Postprocessing
		into vParent, vInheritance, vPreprocessingTSQL, vPostprocessingTSQL, vPreprocessing1, vPostprocessing1
	from RecroSec_ObjectPermission p
	where p.ObjectPermissionId = vObjectPermissionId;
    
	--1. PreprocessingTSQL, módosíthatja a kiindulási ObjectPermissionId-t (fa felépítésű adatok eseten hasznos)
	if( vPreprocessingTSQL is not null) then
        RecroSec_IsObjectValid(vPreprocessingTSQL, 'P', isObjectValid);
		if( isObjectValid = 1) then
            vStorable := 1;
            origObjectPermissionId := vObjectPermissionId;
            vStatement := CONCAT('begin ', CONCAT(vPreprocessingTSQL, '(:vUserId, :OBJECTNAME, :OBJECTKEY, :vObjectPermissionId, :vStorable); end;'));
            EXECUTE IMMEDIATE vStatement USING IN vUserId, IN OBJECTNAME, IN OBJECTKEY, IN OUT vObjectPermissionId, OUT vStorable;

			if( vStorable = 0 ) then
				STORABLE := 0;
            end if;

			if( origObjectPermissionId <> vObjectPermissionId) then
                select count(1) into l_exists from TMP_RECROGRID_INT where Id=vObjectPermissionId;
				if( l_exists = 0 ) then
				   vStatement := 'DECLARE ResA SYS_REFCURSOR; ResB SYS_REFCURSOR;
						begin RECROSEC_GETPERMISSIONFORUSERA(:vUserId,:vObjectPermissionId,:OBJECTNAME,:OBJECTKEY,
							:vStorable,:vPreprocessing1,:vPostprocessing1,:ERRORCODE,:NESTED,null,ResA,ResB); end;';
				   EXECUTE IMMEDIATE vStatement USING IN vUserId, IN vObjectPermissionId, IN OBJECTNAME, IN OBJECTKEY,
						OUT vStorable, OUT vPreprocessing1, OUT vPostprocessing1, OUT ERRORCODE, IN NESTED;

					if( vStorable = 0 ) then
						STORABLE := 0;
                    end if;

					if( PREPROCESSING is null and vPreprocessing1 is not null ) then
						PREPROCESSING := vPreprocessing1;
                    end if;

					if( POSTPROCESSING is null and vPostprocessing1 is not null ) then
						POSTPROCESSING := vPostprocessing1;
                    end if;

					return;
				end if;
                vObjectPermissionId := origObjectPermissionId;
			end if;
		end if;
	end if;
    
	insert into TMP_RECROGRID_INT values(vObjectPermissionId);

    if( PREPROCESSING is null and vPreprocessing1 is not null ) then
        PREPROCESSING := vPreprocessing1;
    end if;

    if( POSTPROCESSING is null and vPostprocessing1 is not null ) then
        POSTPROCESSING := vPostprocessing1;
    end if;
    
	--2. CustomUserPermission összegyűjtése
	insert into TMP_RECROGRID_PERMISSION(PermissionId, Permission)
		select PermissionId, Permission from RecroSec_CustomUserPermission
		where UserId = vUserId and ObjectPermissionId = vObjectPermissionId and PermissionId not in(select Id from TMP_RECROGRID_INT);
    
	--3. UserPermission összegyűjtése priority szerint
    open userPermissionGroup(vUserId, vObjectPermissionId);
    loop
        fetch userPermissionGroup into rec_c1;
        exit when userPermissionGroup%notfound;

		if(Nested = 0 or rec_c1.Inheritable = 1) then
			insert into TMP_RECROGRID_PERMISSION(PermissionId, Permission)
				select p.PermissionId, p.Permission from recrosec_permission p
				where p.PermissionGroupId = rec_c1.PermissionGroupId and p.PermissionId not in(select Id from TMP_RECROGRID_INT);
		end if;
        
    end loop;
    close userPermissionGroup;    
    
	--4. RolePermission összegyűjtése Role priority, azon belül priority szerint
    open rolePermissionGroup(vUserId, vObjectPermissionId);
    loop
        fetch rolePermissionGroup into rec_c2;
        exit when rolePermissionGroup%notfound;

		if(Nested = 0 or rec_c2.Inheritable = 1) then
			insert into TMP_RECROGRID_PERMISSION(PermissionId, Permission)
				select p.PermissionId, p.Permission from recrosec_permission p
				where p.PermissionGroupId = rec_c2.PermissionGroupId and p.PermissionId not in(select Id from TMP_RECROGRID_INT);
		end if;
        
    end loop;
    close rolePermissionGroup;    

	--5. Inheritance eseten parent alapján az 1. ponttól
	if( vInheritance = 1 and vParent is not null) then
		vStorable := STORABLE;
        
		vStatement := 'DECLARE ResA SYS_REFCURSOR; ResB SYS_REFCURSOR;
			begin RECROSEC_GETPERMISSIONFORUSERA(:vUserId,:vObjectPermissionId,:OBJECTNAME,:OBJECTKEY,
				:vStorable,:vPreprocessing1,:vPostprocessing1,:ERRORCODE,1,null,ResA,ResB); end;';
		EXECUTE IMMEDIATE vStatement USING IN vUserId, IN vObjectPermissionId, IN OBJECTNAME, IN OBJECTKEY,
			OUT vStorable, OUT vPreprocessing1, OUT vPostprocessing1, OUT ERRORCODE;

		if( ERRORCODE <> 0 ) then
			return;
		end if;

        if( vStorable = 0 ) then
            STORABLE := 0;
        end if;

        if( PREPROCESSING is null and vPreprocessing1 is not null ) then
            PREPROCESSING := vPreprocessing1;
        end if;

        if( POSTPROCESSING is null and vPostprocessing1 is not null ) then
            POSTPROCESSING := vPostprocessing1;
        end if;
        
	end if;
    
	--6. PostprocessingTSQL a #Permission módosítására
	if( vPostprocessingTSQL is not null) then
        RecroSec_IsObjectValid(vPostprocessingTSQL, 'P', isObjectValid);
		if( isObjectValid = 1) then
            vStorable := 1;
            vStatement := CONCAT('begin ', CONCAT(vPostprocessingTSQL, '(:vUserId, :OBJECTNAME, :OBJECTKEY, :vStorable); end;'));
            EXECUTE IMMEDIATE vStatement USING IN vUserId, IN OBJECTNAME, IN OBJECTKEY, OUT vStorable;

			if( vStorable = 0 ) then
				STORABLE := 0;
            end if;
		end if;
	end if;

    --res    
	if( NESTED = 0 ) then
        select count(1) into l_exists from TMP_RECROGRID_PERMISSION;
		if( l_exists <> 0 ) then
            OPEN RES FOR select p.PermissionId, p.Permission from TMP_RECROGRID_PERMISSION p ORDER BY 1;
		else
            OPEN RES FOR SELECT 0 AS PermissionId, CAST(0 AS NUMBER(1)) AS Permission from DUAL;
		end if;
		OPEN RES2 FOR SELECT ERRORCODE AS ErrorCode, STORABLE AS Storable, PREPROCESSING AS Preprocessing, POSTPROCESSING AS Postprocessing from DUAL;
	end if;

END RECROSEC_GETPERMISSIONFORUSERA;
/

CREATE OR REPLACE PROCEDURE RECROSEC_GETPERMISSIONFORUSER 
(
  USERID IN NVARCHAR2
, OBJECTPERMISSIONID IN INT DEFAULT 0 
, OBJECTNAME IN VARCHAR2 DEFAULT NULL 
, OBJECTKEY IN NVARCHAR2 DEFAULT NULL 
, USERNAME IN VARCHAR2 DEFAULT NULL 
, RES OUT SYS_REFCURSOR
, RES2 OUT SYS_REFCURSOR
) AS 
	vStatement varchar2(2000);
    vStorable int := 0;
	vPreprocessing1 varchar2(255) := null;
	vPostprocessing1 varchar2(255) := null;    
	ERRORCODE INT := 0;
	NESTED INT := 0;
BEGIN

    RECROSEC_GETPERMISSIONFORUSERA(USERID,OBJECTPERMISSIONID,OBJECTNAME,OBJECTKEY,
		vStorable,vPreprocessing1,vPostprocessing1,ERRORCODE,NESTED,USERNAME,RES,RES2);

END RECROSEC_GETPERMISSIONFORUSER;
/
