﻿/*RecroGrid Framework DB v1.4.2 PostgreSQL*/

DO $$
BEGIN
	IF (EXISTS(SELECT * FROM pg_catalog.pg_tables WHERE tablename='aspnetusers') and
		EXISTS(SELECT * FROM pg_catalog.pg_tables WHERE tablename='aspnetroles') and
		EXISTS(SELECT * FROM pg_catalog.pg_tables WHERE tablename='aspnetuserroles')) THEN

		CREATE OR REPLACE VIEW vrecrosec_user
			AS	SELECT id AS userid, UserName AS username FROM aspnetusers;

		CREATE OR REPLACE VIEW vrecrosec_role
			AS SELECT id AS roleid, name AS rolename FROM aspnetroles;

		CREATE OR REPLACE VIEW vrecrosec_usersinrole
			AS SELECT userid AS userid, roleid AS roleid FROM aspnetuserroles;

		INSERT INTO aspnetroles(id, name) select '1', 'Administrators' where NOT EXISTS (SELECT * FROM aspnetroles WHERE id = '1' or name = 'Administrators');
		INSERT INTO aspnetroles(id, name) select '2', 'Users' where NOT EXISTS (SELECT * FROM aspnetroles WHERE id = '2' or name = 'Users');
	END IF;
END;
$$ LANGUAGE plpgsql;
/*$$*/

DO $$
DECLARE
    i  int;
BEGIN

	--ALTER SEQUENCE recrogrid_entity_entityid_seq RESTART WITH 10001;
	SELECT COALESCE(MAX(entityid),0) INTO i FROM recrogrid_entity;
	PERFORM pg_catalog.setval(pg_get_serial_sequence('recrogrid_entity', 'entityid'), GREATEST(i, 10000));

	--ALTER SEQUENCE recrogrid_property_propertyid_seq RESTART WITH 10001;
	SELECT COALESCE(MAX(propertyid),0) INTO i FROM recrogrid_property;
	PERFORM pg_catalog.setval(pg_get_serial_sequence('recrogrid_property', 'propertyid'), GREATEST(i, 10000));

	--ALTER SEQUENCE recrogrid_options_id_seq RESTART WITH 10001;
	SELECT COALESCE(MAX(id),0) INTO i FROM recrogrid_options;
	PERFORM pg_catalog.setval(pg_get_serial_sequence('recrogrid_options', 'id'), GREATEST(i, 10000));

END;
$$ LANGUAGE plpgsql;
/*$$*/

/*************************************************************************************************************************************************************************************/
CREATE OR REPLACE FUNCTION recrogrid_config_update()
 RETURNS TRIGGER AS $$
BEGIN
 IF (TG_TABLE_NAME = 'recrogrid_menu') THEN
   UPDATE recrogrid_framework SET configvalue = to_char(NOW(), 'YYYY-MM-DD HH24:MI:SS.FXMS') WHERE configname = 'RGFMenuLastUpdated';
 ELSIF (TG_TABLE_NAME = 'recrodict_str') THEN
   UPDATE recrogrid_framework SET configvalue = to_char(NOW(), 'YYYY-MM-DD HH24:MI:SS.FXMS') WHERE configname = 'RecroDictLastUpdated';
 ELSE
   UPDATE recrogrid_framework SET configvalue = to_char(NOW(), 'YYYY-MM-DD HH24:MI:SS.FXMS') WHERE configname = 'TSContextLastUpdated';
 END IF;
 RETURN NULL;
END;
$$ LANGUAGE plpgsql;
/*$$*/

DROP TRIGGER IF EXISTS RecroDict_Str_Change ON recrodict_str;
/*$$*/																											
CREATE TRIGGER RecroDict_Str_Change
    AFTER INSERT OR DELETE OR UPDATE ON recrodict_str
    FOR EACH ROW EXECUTE PROCEDURE recrogrid_config_update();
/*$$*/
DROP TRIGGER IF EXISTS RecroGrid_Entity_Change ON recrogrid_entity;
/*$$*/																											
CREATE TRIGGER RecroGrid_Entity_Change
    AFTER INSERT OR DELETE OR UPDATE ON recrogrid_entity
    FOR EACH ROW EXECUTE PROCEDURE recrogrid_config_update();
/*$$*/
DROP TRIGGER IF EXISTS RecroGrid_Property_Change ON recrogrid_property;
/*$$*/																											
CREATE TRIGGER RecroGrid_Property_Change
    AFTER INSERT OR DELETE OR UPDATE ON recrogrid_property
    FOR EACH ROW EXECUTE PROCEDURE recrogrid_config_update();
/*$$*/
DROP TRIGGER IF EXISTS RecroGrid_Options_Change ON recrogrid_options;
/*$$*/																											
CREATE TRIGGER RecroGrid_Options_Change
    AFTER INSERT OR DELETE OR UPDATE ON recrogrid_options
    FOR EACH ROW EXECUTE PROCEDURE recrogrid_config_update();
/*$$*/
DROP TRIGGER IF EXISTS RecroTrack_Table_Change ON recrotrack_table;
/*$$*/																											
CREATE TRIGGER RecroTrack_Table_Change
    AFTER INSERT OR DELETE OR UPDATE ON recrotrack_table
    FOR EACH ROW EXECUTE PROCEDURE recrogrid_config_update();
/*$$*/
DROP TRIGGER IF EXISTS RGFMenu_Change ON recrogrid_menu;
/*$$*/																											
CREATE TRIGGER RGFMenu_Change
    AFTER INSERT OR DELETE OR UPDATE ON recrogrid_menu
    FOR EACH ROW EXECUTE PROCEDURE recrogrid_config_update();
/*$$*/

/*************************************************************************************************************************************************************************************/
CREATE OR REPLACE FUNCTION recrogrid_replaceid(origentityid INT, newentityid INT)
 RETURNS void AS $$
BEGIN

    BEGIN
	INSERT INTO recrogrid_entity
		   (entityid
		   ,entityname
		   ,entityversion
		   ,typedefinition
		   ,category
		   ,title
		   ,entitycomment)
		SELECT 
			newentityid
			,CONCAT('_', e.entityname)
			,e.entityversion
			,e.typedefinition
			,e.category
			,e.title
			,e.entitycomment FROM recrogrid_entity e WHERE e.entityid=origentityid;

	UPDATE recrogrid_property rp SET entityid=newentityid WHERE rp.entityid=origentityid;
	UPDATE recrogrid_property rp SET entitybase=newentityid WHERE rp.entitybase=origentityid;
	UPDATE recrogrid_options o SET entityid=newentityid WHERE o.entityid=origentityid;

	DELETE FROM recrogrid_entity WHERE entityid=origentityid;
	UPDATE recrogrid_entity e SET entityname=SUBSTRING(e.entityname,2) WHERE e.entityid=newentityid;
    END;

END;
$$ LANGUAGE plpgsql;
/*$$*/

/*************************************************************************************************************************************************************************************/
DO $$
BEGIN
IF EXISTS(SELECT * FROM pg_catalog.pg_tables WHERE tablename='vrecrosec_fep') THEN
	DROP TABLE vrecrosec_fep;
END IF;
END;
$$ LANGUAGE plpgsql;
/*$$*/

CREATE OR REPLACE VIEW vrecrosec_fep
AS
	SELECT f.FEPId, EntityId, FEPType, TransactionId, Key1
	FROM recrosec_fep f inner join (
		SELECT MAX(FEPId) as FEPId, Key1
		FROM recrotrack_track rtt 
			inner join recrotrack_object rto ON rtt.ObjectId = rto.ObjectId
			inner join recrosec_fep fep ON rtt.TransactionId = fep.TransactionId
		GROUP BY Key1) t on f.FEPId = t.FEPId
/*$$*/

--RecroSec
DO $$
BEGIN
IF EXISTS(SELECT * FROM pg_catalog.pg_tables WHERE tablename='vrecrosec') THEN
	DROP TABLE vrecrosec;
END IF;
END;
$$ LANGUAGE plpgsql;
/*$$*/

CREATE OR REPLACE VIEW vrecrosec AS
	SELECT objectpermissionid, objectname, objectkey FROM recrosec_objectpermission
/*$$*/

CREATE OR REPLACE VIEW vrecrosec_basepermission AS
	SELECT 
		permissiongroupid,
		MAX(CASE WHEN permissionid = 1 THEN permission::INT ELSE NULL END)::BOOLEAN as createperm,
		MAX(CASE WHEN permissionid = 2 THEN permission::INT ELSE NULL END)::BOOLEAN as readperm,
		MAX(CASE WHEN permissionid = 4 THEN permission::INT ELSE NULL END)::BOOLEAN as updateperm,
		MAX(CASE WHEN permissionid = 8 THEN permission::INT ELSE NULL END)::BOOLEAN as deleteperm,
		CURRENT_DATE as rowtimestamp
	FROM recrosec_permission
	GROUP BY permissiongroupid
/*$$*/

CREATE OR REPLACE FUNCTION fnRecroSec_GetObjectPermission(objectname varchar(255), objectkey varchar(128))
 RETURNS int AS $$
DECLARE
	vObjectName varchar(255) default ObjectName;
	vObjectKey varchar(128) default ObjectKey;
	vObjectPermissionId int default null;
	idx int;
BEGIN

    SELECT STRPOS(vObjectName, '/') INTO idx;
	IF( idx > 0 and COALESCE(vObjectKey,'') ~ '^[0-9]+$' ) THEN
		SELECT p.ObjectPermissionId INTO vObjectPermissionId
			FROM recrosec_objectpermission p
			WHERE p.ObjectName = vObjectName and p.Parent = cast(vObjectKey as int);

		IF( vObjectPermissionId is not null ) THEN
			RETURN vObjectPermissionId;
		END IF;

		IF( vObjectKey <> '0' ) THEN
			RETURN cast(vObjectKey as int);
		END IF;

		--parent keresés
		SELECT SUBSTRING(vObjectName, 1, idx - 1) INTO vObjectName;
		SELECT fnRecroSec_GetObjectPermission(vObjectName, '') INTO vObjectPermissionId;
		RETURN vObjectPermissionId;
	END IF;

	IF( COALESCE(vObjectKey,'') <> '' ) THEN
		SELECT p.ObjectPermissionId INTO vObjectPermissionId
			FROM recrosec_objectpermission p
			WHERE p.ObjectName = vObjectName and p.ObjectKey = vObjectKey;

		IF( vObjectPermissionId is not null ) THEN
			RETURN vObjectPermissionId;
		END IF;
	END IF;

	SELECT p.ObjectPermissionId INTO vObjectPermissionId
		FROM recrosec_objectpermission p
		WHERE p.ObjectName = vObjectName and p.ObjectKey is null;
					
	RETURN vObjectPermissionId;

END
$$ LANGUAGE plpgsql;
/*$$*/

CREATE OR REPLACE FUNCTION recrosec_isobjectvalid(
	"name" varchar(255),
	"type" varchar(2))
 RETURNS boolean AS $$
BEGIN
	RETURN true;
END
$$ LANGUAGE plpgsql;
/*$$*/

CREATE OR REPLACE FUNCTION recrosec_getobjectpermission(refcursor, objectname varchar(255), objectkey varchar(128))
 RETURNS refcursor AS $$
DECLARE
 vObjectPermissionId int;																														
BEGIN
	SELECT fnrecrosec_getobjectpermission(objectname, objectkey) INTO vObjectPermissionId;
	OPEN $1 FOR SELECT * FROM recrosec_objectpermission WHERE objectpermissionid = vObjectPermissionId;
	RETURN $1;
END
$$ LANGUAGE plpgsql;
/*$$*/

DROP TABLE IF EXISTS rgf_tmpPermission;
DROP TABLE  IF EXISTS rgf_tmpSkip;
CREATE OR REPLACE FUNCTION recrosec_getpermissionforusera(refcursor,
	UserId varchar(128)
	,ObjectPermissionId int
	,ObjectName varchar(255)
	,ObjectKey varchar(128)
	,Nested int default 0
	,UserName varchar(255) default null
	,OUT Storable boolean
	,OUT Preprocessing varchar(255)
	,OUT Postprocessing varchar(255)
	,OUT ErrorCode int)
 AS $$									
DECLARE
	res ALIAS FOR $1;

	vUserId varchar(128) := UserId;
	vUserName varchar(255) := UserName;
    vObjectPermissionId int := ObjectPermissionId;
	origObjectPermissionId int;

	isObjectValid boolean;
	vStatement varchar(2000);
    vStorable boolean := false;
	
	vParent int := null;
	vInheritance boolean := false;
	vPreprocessingTSQL varchar(255) := null;
	vPostprocessingTSQL varchar(255) := null;
	vPreprocessing varchar(255) := null;
	vPostprocessing varchar(255) := null;    

    userPermissionGroup CURSOR (vUserId varchar(255), vObjectPermissionId int) FOR
		SELECT g.permissionGroupId,op.inheritable
		FROM recrosec_permissiongroup g inner join
			 recrosec_oppg op on op.PermissionGroupId=g.PermissionGroupId inner join
			 recrosec_userpermission u on u.PermissionGroupId=g.PermissionGroupId
		WHERE u.UserId=vUserId and op.ObjectPermissionId=vObjectPermissionId
		ORDER BY op.Priority;
	rec_c1 record;

    rolePermissionGroup CURSOR (vUserId varchar(255), vObjectPermissionId int) FOR
		SELECT g.permissionGroupId,op.inheritable
		FROM recrosec_permissiongroup g inner join
			 recrosec_oppg op on op.PermissionGroupId=g.PermissionGroupId inner join
			 recrosec_rolepermission r on g.PermissionGroupId=r.PermissionGroupId inner join
			 vrecrosec_usersinrole ur on r.RoleId=ur.RoleId left outer join
			 RecroSec_RolePriority rp on r.RoleId=rp.RoleId
		WHERE ur.UserId=vUserId and op.ObjectPermissionId=vObjectPermissionId
		ORDER BY COALESCE(rp.Priority,2147483647), op.Priority;        
    rec_c2 record;

BEGIN
	ErrorCode := 0;

	IF( COALESCE(vObjectPermissionId,0) = 0 AND ObjectName IS NOT NULL ) THEN
		SELECT fnRecroSec_GetObjectPermission(ObjectName, ObjectKey) INTO vObjectPermissionId;
	END IF;

	IF( COALESCE(vObjectPermissionId,0) = 0 ) THEN
		ErrorCode := -1;
		IF(Nested = 0) THEN
			Storable := false;
			Preprocessing := '';
			Postprocessing := '';
			OPEN res FOR SELECT 0 AS PermissionId, false AS Permission;
		END IF;
		RETURN;
	END	IF;

	IF( Nested = 0 ) THEN

		IF( vUserId is null ) THEN
			SELECT UserId FROM vRecroSec_User WHERE UserName = vUserName INTO vUserId;
		END IF;

		Storable := true;
		Preprocessing := null;
		Postprocessing := null;

		CREATE TEMP TABLE IF NOT EXISTS rgf_tmpPermission(
			PermissionId int NOT NULL,
			Permission boolean NOT NULL,
			PRIMARY KEY (PermissionId));

		CREATE TEMP TABLE IF NOT EXISTS rgf_tmpSkip(ObjectPermissionId int NOT NULL);
	END IF;

	SELECT p.Parent, p.Inheritance, p.PreprocessingTSQL,  p.PostprocessingTSQL, p.Preprocessing,  p.Postprocessing
		into vParent, vInheritance, vPreprocessingTSQL, vPostprocessingTSQL, vPreprocessing, vPostprocessing
	FROM recrosec_objectpermission p
	WHERE p.ObjectPermissionId = vObjectPermissionId;

	--1. PreprocessingTSQL, módosíthatja a kiindulási ObjectPermissionId-t (fa felépítésű adatok eseten hasznos)
	IF( COALESCE(vPreprocessingTSQL,'') <> '') THEN
        SELECT recrosec_isobjectvalid(vPreprocessingTSQL, 'P') INTO isObjectValid;
		IF( isObjectValid = true) THEN
			origObjectPermissionId := vObjectPermissionId;
			vStatement := CONCAT('select * from ', vPreprocessingTSQL, '($1,$2,$3,$4)');
			EXECUTE vStatement INTO vObjectPermissionId, vStorable USING vUserId, ObjectName, ObjectKey, vObjectPermissionId;

			IF( vStorable = false ) THEN
            	Storable := false;
        	END IF;

			IF( origObjectPermissionId <> vObjectPermissionId) THEN
				if( not exists(SELECT * FROM rgf_tmpSkip s where s.ObjectPermissionId=vObjectPermissionId) ) then
					SELECT * FROM recrosec_getpermissionforusera(res,
							vUserId, vObjectPermissionId, ObjectName, ObjectKey, Nested, UserName)
							into vStorable, vPreprocessing, vPostprocessing, ErrorCode;

					IF( vStorable = false ) THEN
						Storable := false;
					END IF;

					IF( Preprocessing is null and COALESCE(vPreprocessing,'') <> '') THEN
						Preprocessing := vPreprocessing;
					END IF;

					IF( Postprocessing is null and COALESCE(vPostprocessing,'') <> '' ) THEN
						Postprocessing := vPostprocessing;
					END IF;

					RETURN;
				END IF;
                vObjectPermissionId := origObjectPermissionId;
			END IF;

		END IF;
	END IF;

	INSERT INTO rgf_tmpSkip VALUES(vObjectPermissionId);

    IF( Preprocessing is null and COALESCE(vPreprocessing,'') <> '' ) THEN
        Preprocessing := vPreprocessing;
    END IF;

    IF( Postprocessing is null and COALESCE(vPostprocessing,'') <> '' ) THEN
        Postprocessing := vPostprocessing;
    END IF;

	--2. CustomUserPermission összegyűjtése
	INSERT INTO rgf_tmpPermission(PermissionId, Permission)
		SELECT p.permissionid, p.permission FROM recrosec_customuserpermission p
		WHERE p.userid = vUserId and p.ObjectPermissionId = vObjectPermissionId and p.permissionId not in(SELECT PermissionId FROM rgf_tmpPermission);

	--3. UserPermission összegyűjtése priority szerint
	FOR rec_c1 IN userPermissionGroup(vUserId, vObjectPermissionId) LOOP
		IF(Nested = 0 or rec_c1.Inheritable = true) THEN
			INSERT INTO rgf_tmpPermission(PermissionId, Permission)
				SELECT p.PermissionId, p.Permission FROM recrosec_permission p
				WHERE p.PermissionGroupId = rec_c1.PermissionGroupId and p.PermissionId not in(SELECT PermissionId FROM rgf_tmpPermission);
		END IF;
	END LOOP;
						 
	--4. RolePermission összegyűjtése priority szerint
	FOR rec_c2 IN rolePermissionGroup(vUserId, vObjectPermissionId) LOOP
		IF(Nested = 0 or rec_c2.Inheritable = true) THEN
			INSERT INTO rgf_tmpPermission(PermissionId, Permission)
				SELECT p.PermissionId, p.Permission FROM recrosec_permission p
				WHERE p.PermissionGroupId = rec_c2.PermissionGroupId and p.PermissionId not in(SELECT PermissionId FROM rgf_tmpPermission);
		END IF;
	END LOOP;

	--5. Inheritance eseten parent alapján az 1. ponttól
	IF( vInheritance = true and vParent is not null) THEN
		vStorable := Storable;
        
		SELECT recrosec_getpermissionforusera(NULL,
				vUserId, vObjectPermissionId, vObjectName, vObjectKey, 1)
				into vStorable, vPreprocessing, vPostprocessing, ErrorCode;
						 
		IF( ErrorCode <> 0 ) THEN
			RETURN;
		END IF;

        IF( vStorable = false ) THEN
            Storable := false;
        END IF;

        IF( Preprocessing is null and COALESCE(vPreprocessing,'') <> '' ) THEN
            Preprocessing := vPreprocessing;
        END IF;

        IF( Postprocessing is null and COALESCE(vPostprocessing,'') <> '' ) THEN
            Postprocessing := vPostprocessing;
        END IF;
        
	END IF;

	--6. PostprocessingTSQL a #Permission módosítására
	IF( COALESCE(vPostprocessingTSQL,'') <> '') THEN
        SELECT recrosec_isobjectvalid(vPostprocessingTSQL, 'P') INTO isObjectValid;
		IF( isObjectValid = true) THEN
			vStatement := CONCAT('select ', vPostprocessingTSQL, '($1,$2,$3)');
			EXECUTE vStatement into vStorable USING vUserId, ObjectName, ObjectKey;

			IF( vStorable = false ) THEN
				Storable := false;
			END IF;
		END IF;
	END IF;

	--res
	IF( Nested = 0 ) THEN
		IF( exists(SELECT * FROM rgf_tmpPermission) ) THEN
            OPEN res FOR SELECT p.PermissionId, p.Permission FROM rgf_tmpPermission p ORDER BY 1;
		ELSE
            OPEN res FOR SELECT 0 AS PermissionId, false AS Permission;
		END IF;
	END IF;

END;
$$ LANGUAGE plpgsql;
/*$$*/

CREATE OR REPLACE FUNCTION recrosec_getpermissionforuser(refcursor,refcursor,
	UserId varchar(128)
	,ObjectPermissionId int default 0
	,ObjectName varchar(255) default null
	,ObjectKey varchar(128) default null
	,UserName varchar(255) default null)
 RETURNS SETOF refcursor AS $$
DECLARE
	Storable boolean;
	Preprocessing varchar(255);
	Postprocessing varchar(255);
	ErrorCode int;
BEGIN

	SELECT * from recrosec_getpermissionforusera($1,
				UserId, ObjectPermissionId, ObjectName, ObjectKey, 0, UserName)
				into Storable, Preprocessing, Postprocessing, ErrorCode;
	RETURN NEXT $1;

	OPEN $2 FOR SELECT ErrorCode, Storable, Preprocessing, Postprocessing;
	RETURN NEXT $2;

END;
$$ LANGUAGE plpgsql;
/*$$*/
