namespace $rootnamespace$.DBModel
{
    using Microsoft.EntityFrameworkCore;
    using System;
    using System.ComponentModel.DataAnnotations.Schema;
    using System.Linq;

    //Scaffold-DbContext "Server=(localdb)\mssqllocaldb;Database=RGFDemoCore;Trusted_Connection=True;" Microsoft.EntityFrameworkCore.SqlServer -o DBModel
    //Scaffold-DbContext "Host=postgresql.lan;Database=RGFDemoCore;Username=recrogrid;Password=P@ssw0rd" Npgsql.EntityFrameworkCore.PostgreSQL -o DBModel
    //Scaffold-DbContext "Data Source=Oracle12c.lan:1521/ora12;User Id=C##RGF_USER_CORE;Password=P@ssw0rd;Persist Security Info=True;" Oracle.EntityFrameworkCore -o DBModel
    //Add-Migration InitialCreate -Context DatabaseContext -OutputDir DBModel\Migrations

    public partial class DatabaseContext : DbContext
    {
        public DatabaseContext() { }
        public DatabaseContext(DbContextOptions<DatabaseContext> options) : base(options) { }

        #region DbSet
        #endregion

        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            if (!optionsBuilder.IsConfigured)
            {
#if Oracle
                //Oracle
                optionsBuilder.UseOracle(Recrovit.RecroGridFramework.Data.RGDataContext.DefaultConnectionString,
                    opts => opts.CommandTimeout(Recrovit.RecroGridFramework.Data.RGDataContext.SQLTimeout));
#elif PGSQL
                //PostgreSQL
                optionsBuilder.UseNpgsql(Recrovit.RecroGridFramework.Data.RGDataContext.DefaultConnectionString,
                    opts => opts.CommandTimeout(Recrovit.RecroGridFramework.Data.RGDataContext.SQLTimeout));
#else
                optionsBuilder.UseSqlServer(Recrovit.RecroGridFramework.Data.RGDataContext.DefaultConnectionString,
                    opts => opts.CommandTimeout(Recrovit.RecroGridFramework.Data.RGDataContext.SQLTimeout));
#endif
            }
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            //base.OnModelCreating(modelBuilder);
#if Oracle
            //Oracle
            var builder = new System.Data.Common.DbConnectionStringBuilder();
            builder.ConnectionString = this.Database.GetDbConnection().ConnectionString;
            string schemaName = (builder["User ID"] as string).ToUpper();

            modelBuilder.HasDefaultSchema(schemaName);
            var types = modelBuilder.Model.GetEntityTypes().ToList();
            types.ForEach(e => e.Relational().TableName = e.Relational().TableName.ToUpper());
            types.SelectMany(e => e.GetProperties()).ToList().ForEach(p => p.Relational().ColumnName = p.Relational().ColumnName.ToUpper());

#elif PGSQL
            //PostgreSQL
            modelBuilder.HasDefaultSchema("public");
            var types = modelBuilder.Model.GetEntityTypes().ToList();
            types.ForEach(e => e.Relational().TableName = e.Relational().TableName.ToLower());
            types.SelectMany(e => e.GetProperties()).ToList().ForEach(p => p.Relational().ColumnName = p.Relational().ColumnName.ToLower());
#endif
            Recrovit.RecroGridFramework.Data.Core.RecroGridDbContext.OnModelCreating(this, modelBuilder);
        }
    }
}
